import sys
import asyncio
import threading
import time
import urllib.request
import logging, logging.handlers
import os
from pathlib import Path
from .sensing_mgr.const import Const
from .sensing_mgr.io.led_device import LedDevice 
from .cloud_agent.cloud_agent import CloudAgent

class GwApp:
    def __init__(self, loop, dev_cls, config_cls, config_path, cloud_agent_file_path):
        model_config = config_cls()
        if not model_config.load(config_path):
            return

        conf = model_config.get_log_conf()
        self.init_logging(conf.get(model_config.FILE), conf.get(model_config.STREAM))
        self._logging = logging.getLogger(Const.LOGGER_NAME)

        conf = model_config.get_default_conf()
        cloud_config = conf.get(model_config.CLOUD_CONFIG)
        send_cloud = conf.get(model_config.SEND_CLOUD)
        valid_cache = conf.get(model_config.CACHE)

        model_device   = dev_cls(model_config, cloud_config, send_cloud)
        device_id = os.environ.get('DEVINFO_SN')
        if send_cloud:
            cloud_agent = CloudAgent(loop, cloud_agent_file_path, device_id)
        else:
            cloud_agent = None

        model_device.setup_reporters(model_config, cloud_agent, valid_cache, loop)
 
        self._device_id = device_id
        self._cloud_config = cloud_config
        self._send_cloud = send_cloud
        self._model_config = model_config
        self._model_device   = model_device
        self._cloud_agent = cloud_agent

    def init_logging(self, file, stream):
        LOG_FILE_PATH = Const.LOG_FILE_PATH
        # logging
        formatter = logging.Formatter(
            '%(asctime)s <%(levelname)s> : %(message)s')

        logger = logging.getLogger(Const.LOGGER_NAME)
        logger.setLevel(logging.DEBUG)

        if not os.path.exists(LOG_FILE_PATH):
            file = open(LOG_FILE_PATH, "w")
            file.write("")
            file.close()

        if file is True:
            filehandler = logging.handlers.RotatingFileHandler(
                LOG_FILE_PATH, encoding='utf-8',
                maxBytes=10000, backupCount=10)
            filehandler.setFormatter(formatter)
            logger.addHandler(filehandler)

        if stream is True:
            streamhandler = logging.StreamHandler()
            streamhandler.setFormatter(formatter)
            logger.addHandler(streamhandler)

    def is_data_oneshot_stop(self):
        conf = self._model_config.get_default_conf()
        return conf.get(self._model_config.DATA_SEND_ONESHOT)

    def wait_container_stop_interval(self):
        conf = self._model_config.get_default_conf()
        return conf.get(self._model_config.WAIT_CONTAINER_STOP)

    async def oneshot(self, stop_interval):
        self._threads = []
        max_interval = 1 
        send_margin = 5
        self._logging.info("container exec data_oneshot_stop.")

        for loopable in self._model_device.loopables():
            thread = threading.Thread(target=loopable.start_loop)
            interval = loopable.oneshot()
            if max_interval < interval:
                max_interval = interval
            thread.start()
            self._threads.append(thread)
        start_time = time.time()

        while(time.time() - start_time) < (max_interval + send_margin):
            await asyncio.sleep(1)

        start_time = time.time()
        
        while (time.time() - start_time) < stop_interval:
            await asyncio.sleep(1)

        return

    async def send_deviceinformation(self):
        send_prop = {}
        device_data = {}
        device_data["DevInfo_SerialNumber"] = self._device_id
        #device_data["DevInfo_IMEI"] = os.environ.get('DEVINFO_IMEI') 
        device_data["DevInfo_LAN_MAC_Addr"] = os.environ.get('DEVINFO_LAN_MAC') 
        device_data["DevInfo_ABOS_Ver"] = os.environ.get('DEVINFO_ABOS_VER')
        device_data["DevInfo_Container_Ver"] = os.environ.get('DEVINFO_CONTAINER_IMG_VER')
        send_prop["property"] = device_data
        return await self._cloud_agent.send(send_prop)

    async def ready(self):
        if self._send_cloud:
            if await self._cloud_agent.connect():
                with LedDevice.get("app") as led:
                    led.on_or_off(True)
            self._logging.info("send device information to cloud.")
            if not await self.send_deviceinformation():
                with LedDevice.get("app") as led:
                    led.blink_fast()
        return True

    def start(self):
        self._threads = []

        for loopable in self._model_device.loopables():
            thread = threading.Thread(target=loopable.start_loop)
            thread.start()
            self._threads.append(thread)
        return True

    def stop_threads(self):
        for loopable in self._model_device.loopables():
            loopable.request_stop()
        for thread in self._threads:
            thread.join()

    async def shutdown(self):
        self._doCheckConnection = False
        self.stop_threads()
        self._logging.info("all threads are quited, then shutdown the IoT client...")
        await self._cloud_agent.disconnect()


def stdin_listener():
    while True:
        selection = input("Press Q to quit\n")
        if (selection == 'Q' or selection == 'q'):
            print("Quitting...")
            break

async def mainloop(dev_cls, config_cls, config_path, cloud_agent_file_path):
    is_deamon = False
    args=sys.argv

    with LedDevice.get("app") as led:
        led.blink_slow()

    if len(args) == 2:
        if args[1] == "-d":
            is_deamon = True

    loop = asyncio.get_event_loop()
    app = GwApp(loop, dev_cls, config_cls, config_path, cloud_agent_file_path)

    if not await app.ready():
        return

    if app.is_data_oneshot_stop() is True:
        await app.oneshot(app.wait_container_stop_interval())
        await app.shutdown()
        return

    if not app.start():
        return

    if not is_deamon:
        user_finished = loop.run_in_executor(None, stdin_listener)
        await user_finished
        await app.shutdown()
    else:
        while True:
            await asyncio.sleep(10)

#
# End of File
#
